# Build EC-EARTH on LUCIA

EC-Earth is a global coupled climate model. It is developed by a consortium of European research institutions.
The goal of this project is to adapt this model for our needs in ELIC: install, run management, etc.

## User requirements

First, load the required env modules

```bash
module purge
module load EasyBuild/2023a
module load Perl-bundle-CPAN/5.36.1-GCCcore-12.3.0
MODULEPATH=$MODULEPATH:/gpfs/projects/acad/ecearth/softs/easybuild/modules/all
module load netCDF-Fortran/4.6.1-iompi-2023a
module load imkl/2023.1.0 
module load grib_api/1.24.0-iompi-2023a 
module load CDO/1.9.10-iompi-2023a
module load OpenJPEG/2.5.0-GCCcore-12.3.0
```

Then, clone this EC-Earth fork repository (3.3.3.2)

```bash
mkdir -p ~/models
cd ~/models
git clone https://gogs.elic.ucl.ac.be/pbarriat/ecearth3.git ecearth_3.3.3.2
```

## Compile

* Compile **oasis**:

```bash
cd ~/models/ecearth_3.3.3.2/sources/oasis3-mct/util/make_dir
make BUILD_ARCH=ecconf -f TopMakefileOasis3
```

* Compile **IFS**:

```bash
cd ~/models/ecearth_3.3.3.2/sources/ifs-36r4
./makeifs -j 4
```

* Compile **xios**:

```bash
cd ~/models/ecearth_3.3.3.2/sources/xios-2.5
./make_xios --prod --arch ecconf --use_oasis oasis3_mct --netcdf_lib netcdf4_par -j 4
```

* Compile **NEMO**:

```bash
cd ~/models/ecearth_3.3.3.2/sources/nemo-3.6/CONFIG
./makenemo -n ORCA1L75_LIM3 -m ecconf -j4
```

* Compile **runoff-mapper**:

```bash
cd ~/models/ecearth_3.3.3.2/sources/runoff-mapper/src
make
```

## Run

Check the `ecconf.cfg` the first job script `EC00.sh` and submit

```bash
cd ~/models/ecearth_3.3.3.2/runtime/classic
sbatch EC00.sh
```

### Data

I prepared all the input files in `/gpfs/scratch/acad/ecearth/data/bsc32/` directory.

### Very first try

|  CC   | Release | RES |  XIOS  | NEMO | IFS | #NODES |  WTIME per YEAR        |
| ----- | --- | ---- | ------ | -------------- | ------ | ------- | -------------------- |
| `EC00.sh` | iompi/2022.05 | T255L91_ORCA1L75 |  1  |   86  |   320    |     3      | **121min**       |
| `EC01.sh` | iompi/2022.05 | T255L91_ORCA1L75 |  1  |   98  |   392   |    4       | **??min**       |
| `EC02.sh` | iompi/2022.05 | T255L91_ORCA1L75 |  1  |  128  |   512   |    6       | **83min**       |

- 1 year (1850), restart every year

## Contributors

- ELIC members [@pbarriat](https://gogs.elic.ucl.ac.be/pbarriat)
