SUBROUTINE UPDRLXSM(PTSTEP)

!**** *UPDRLXSM*

!     PURPOSE.
!     --------

!     Updates the reference fields used for the nuding of the soil moisture

!**   INTERFACE.
!     ----------

!     CALL UPDRLXSM(PTSTEP)

!        Explicit arguments :
!        --------------------
!        PTSTEP: TIME STEP

!     METHOD.
!     -------

!     Reads the time-varying reference fields (currently soil moisture 
!     for the 4 levels)
!     Only one option is possible: 
!     time interpolation, the input fields are interpolated linearly 
!     between the two nearest points in time

!     EXTERNALS.
!     ----------

!     UPDCALSEC

!     AUTHORS.
!     --------

!         W. MAY (CEC,LU) OCTOBER 2017 (based on I. RINGGAARD's subroutine UPDRLXSI
!         and M. DEQUE's routine UPDCLI, respectively),
!         slightly modified JANUARY 2018 (removing the variables in YOMMCC)

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM, JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMGEM   , ONLY : NGPTOTMX ,&
 & NGPTOTG
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMMP    , ONLY : MYPROC
USE YOMRIP   , ONLY : NINDAT   ,NSTADD   ,NSSSSS   ,NSTASS
USE YOMDYN   , ONLY : TSTEP
USE MPL_MODULE, ONLY : MPL_BROADCAST
USE YOMRLXSM , ONLY : LRXSM, LRXSMT1, LRXSMT2, LRXSMT3, LRXSMT4, LRXSMS, &
 & ZRLX039, ZRLX040, ZRLX041, ZRLX042, RLXSMDIR

USE CPLNG

USE NETCDF

IMPLICIT NONE

#include "abor1.intfb.h"
#include "ircvgpf.intfb.h"
#include "isndgpf.intfb.h"
#include "updcalsec.intfb.h"

REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTEP 
INTEGER(KIND=JPIM) :: ILMOIS(12)
REAL(KIND=JPRB) :: ZBUF (NGPTOTG)
REAL(KIND=JPRB) :: ZFLD (NGPTOTMX)

INTEGER(KIND=JPIM) :: IA0, IAN,&
 & IJ0, IJOUR, &
 & IM0, IMOIS, &
 & ITAG, &
 & IH0, IHOUR, IMIN, ISEC, IDATEC, IDATECH, IDATE1, IDATE2, &
 &  INGPT, IH1, IH2, IA2, IM2, IJ2, &
 & IDMON (12), ILEAP, JDMONTH, NSTASSX

REAL(KIND=JPRB) :: RDATEC, RDATE1, RDATE2, RDIFFC1, RDIFF2C, RDIFF21, &
 & FRHOU, FRMIN, FRSEC 
REAL(KIND=JPRB) :: ZRLX(NGPTOTG), ZRLXP1(NGPTOTG), ZRLXM1(NGPTOTG)

REAL(KIND=JPRB) :: YRLX039 (NGPTOTG), YRLX040 (NGPTOTG), &
 & YRLX041 (NGPTOTG), YRLX042 (NGPTOTG) 

REAL(KIND=JPRB) :: ZFLD039 (NGPTOTMX), ZFLD040 (NGPTOTMX), &
 & ZFLD041 (NGPTOTMX), ZFLD042 (NGPTOTMX) 

INTEGER :: INCID
INTEGER :: INCRESULT

CHARACTER (LEN=*), PARAMETER :: NUM_039_NAME='var39'
CHARACTER (LEN=*), PARAMETER :: NUM_040_NAME='var40'
CHARACTER (LEN=*), PARAMETER :: NUM_041_NAME='var41'
CHARACTER (LEN=*), PARAMETER :: NUM_042_NAME='var42'

INTEGER :: NUM_039_VARID
INTEGER :: NUM_040_VARID
INTEGER :: NUM_041_VARID
INTEGER :: NUM_042_VARID

CHARACTER :: CRLXHOU1*2
CHARACTER :: CRLXDAT1*8
CHARACTER :: CRLXHOU2*2
CHARACTER :: CRLXDAT2*8

CHARACTER :: CRLN039*200
CHARACTER :: CRLN040*200
CHARACTER :: CRLN041*200
CHARACTER :: CRLN042*200

!
! STEERING THE NUDGING OF SOIL MOISTURE 
!
! LRXSM: PARAMETER INDICATING THE LEVELS TO BE NUDGED 
!
! LRXSM =  0: NO NUDGING (SUBROUTINE IS NOT ENTERED)      
! LRXSM = 12: 4XDAILY DATA AND 3 LEVELS (EXCLUDING LEVEL 1)      
! LRXSM = 13: 4XDAILY DATA AND 4 LEVELS
!
! LRXSMT: PARAMETER INDICATING THE TIME SCALE OF THE RELAXATION (IN HOURS)
!  
! LRXSMT =   0: ACTUAL VALUES RELPACED BY EXTERNAL ONES 
! LRXSMT =  24: 1 DAY
! LRXSMT = 120: 5 DAYS
! 
! THE VALUES ARE SPECIFIED FOR EACH LEVEL SEPARATELY 
!
! LRXSMS: PARAMETER INDICATING WHEN THE RELAXATION IS DONE 
!
! LRXSMS = 0: RELAXATION BEFORE THE TIME STEP
! LRXSMS = 1: RELAXATION AFTER THE TIME STEP
!

INTEGER(KIND=JPIM),SAVE :: IDATEREF=0
INTEGER(KIND=JPIM) :: IDATE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcttim.h"

DATA IDMON /31,28,31,30,31,30,31,31,30,31,30,31/

!*
!     1. SETTING CONSTANT VALUES.
!     ---------------------------

IF (LHOOK) CALL DR_HOOK('UPDRLXSM',0,ZHOOK_HANDLE)

!*    1.1 Calendar

IJ0=NDD(NINDAT)
IM0=NMM(NINDAT)
IA0=NCCAA(NINDAT)
IH0=NSSSSS/3600_JPIM

IF ( LRXSMS.EQ.0 ) THEN
   NSTASSX = NSTASS-NINT(TSTEP)/2
ELSEIF ( LRXSMS.EQ.1 ) THEN
   NSTASSX = NSTASS+NINT(TSTEP)/2
ELSE
   NSTASSX = NSTASS
ENDIF

CALL UPDCALSEC(IH0,IJ0,IM0,IA0,NSTADD,NSTASSX,IHOUR,IMIN,ISEC,IJOUR,IMOIS,IAN,ILMOIS,NULOUT)

! Intial date and time 
IDATEC = IJOUR+100*IMOIS+10000*IAN

FRSEC = FLOAT(ISEC)/60.
FRMIN = (FLOAT(IMIN)+FRSEC)/60.
FRHOU = (FLOAT(IHOUR)+FRMIN)/24.
RDATEC = FLOAT(IDATEC)+FRHOU

IDATECH = 100*IDATEC+NINT(FRHOU)

!*
!     2. READ THE SOIL MOISTURE DATA, IF NUDGING FIELDS ARE REQUIRED
!     --------------------------------------------------------------

LRXSMIF: IF (LRXSM > 0) THEN

  MYPROCSMIF: IF (MYPROC == 1) THEN

!   ONLY PROCESSOR 1 SHOULD READ 

    WRITE (CRLXDAT1,'(I8.8)') IDATEC

    IF ( MOD(IHOUR,6).EQ.0 ) THEN
       IH1 = IHOUR
    ELSE
       IF ( IHOUR.LT.6 ) THEN
          IH1 = 0
       ELSEIF ( (IHOUR.GT.6).AND.(IHOUR.LT.12) ) THEN
          IH1 = 6
       ELSEIF ( (IHOUR.GT.12).AND.(IHOUR.LT.18) ) THEN
          IH1 = 12
       ELSE
          IH1 = 18
       ENDIF
    ENDIF

    WRITE (CRLXHOU1,'(I2.2)') IH1

! READ SOIL MOISTURE LEVEL 1 AT T1

    CRLN039 = TRIM(RLXSMDIR) // &
         & '/eraint_land_ana_'//CRLXDAT1//'_'//CRLXHOU1// &
         & 'H_039.nc'

    INCRESULT = NF90_OPEN(CRLN039,NF90_NOWRITE,INCID)
                      
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS OPENING ', CRLN039, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM IN NF90_OPEN')
    ENDIF

    INCRESULT = NF90_INQ_VARID(INCID,NUM_039_NAME,NUM_039_VARID)
             
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR ID', CRLN039, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_INQ_VARID')
    ENDIF

    INCRESULT = NF90_GET_VAR(INCID,NUM_039_VARID,ZRLX, &
              & START=(/1,1/),COUNT=(/NGPTOTG,1/))             

    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR', CRLN039, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_GET_VAR')
    ENDIF

    INCRESULT = NF90_CLOSE(INCID)

    DO INGPT=1,NGPTOTG
       ZRLXM1(INGPT) = ZRLX(INGPT)
       IF (ZRLXM1(INGPT) <= 0.0_JPRB) ZRLXM1(INGPT)=0.0_JPRB 
    ENDDO
         
! HANDLE DATE/TIME
         
    IDATE1 = IDATEC

    IH2 = IH1+6

    IF ( MOD(IAN,4).EQ.0 ) THEN
       ILEAP = 1
    ELSE
       ILEAP = 0
    ENDIF

    IF ( IH2.EQ.24 ) THEN

       IH2 = 0
       IJ2 = IJOUR+1
       
       JDMONTH = IDMON(IMOIS)
       IF ( (ILEAP.EQ.1).AND.(IMOIS.EQ.2) ) JDMONTH = JDMONTH+1           

       IF ( IJ2.GT.JDMONTH ) THEN

          IJ2 = 1
          IM2 = IMOIS+1
    
          IF ( IM2.GT.12 ) THEN
             IM2 = 1
             IA2 = IAN+1
          ELSE
             IA2 = IAN
          ENDIF

       ELSE
          IM2 = IMOIS
          IA2 = IAN
       ENDIF
       
       IDATE2 = IJ2+100*IM2+10000*IA2

    ELSE
       IDATE2 = IDATE1
    ENDIF

    WRITE (CRLXDAT2,'(I8.8)') IDATE2
    WRITE (CRLXHOU2,'(I2.2)') IH2
    
! READ SOIL MOISTURE LEVEL 1 AT T2
         
    CRLN039 = TRIM(RLXSMDIR) // &
            & '/eraint_land_ana_'//CRLXDAT2//'_'//CRLXHOU2// &
            & 'H_039.nc'
 
    INCRESULT = NF90_OPEN(CRLN039,NF90_NOWRITE,INCID)
    
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS OPENING ', CRLN039, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM IN NF90_OPEN')
    ENDIF

    INCRESULT = NF90_INQ_VARID(INCID,NUM_039_NAME,NUM_039_VARID)
        
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR ID', CRLN039, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_INQ_VARID')
    ENDIF

    INCRESULT = NF90_GET_VAR(INCID,NUM_039_VARID,ZRLX, &
              & START=(/1,1/),COUNT=(/NGPTOTG,1/))             

    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR', CRLN039, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_GET_VAR')
    ENDIF

    INCRESULT = NF90_CLOSE(INCID)

    DO INGPT=1,NGPTOTG
       ZRLXP1(INGPT) = ZRLX(INGPT)
       IF (ZRLXP1(INGPT) <= 0.0_JPRB) ZRLXP1(INGPT)=0.0_JPRB 
    ENDDO

    FRHOU = FLOAT(IH1)/24.
    RDATE1 = FLOAT(IDATE1)+FRHOU

    FRHOU = FLOAT(IH2)/24.
    RDATE2 = FLOAT(IDATE2)+FRHOU

    RDIFFC1 = RDATEC-RDATE1
    RDIFF2C = RDATE2-RDATEC   
    RDIFF21 = RDATE2-RDATE1

    DO INGPT=1,NGPTOTG
       YRLX039(INGPT) & 
            & = (RDIFF21-RDIFFC1)*ZRLXM1(INGPT)/RDIFF21 &
            &    +(RDIFF21-RDIFF2C)*ZRLXP1(INGPT)/RDIFF21
       IF (YRLX039(INGPT) <= 0.0_JPRB) YRLX039(INGPT)=0.0_JPRB 
    ENDDO
    
! READ SOIL MOISTURE LEVEL 2 AT T1

    CRLN040 = TRIM(RLXSMDIR) // &
            & '/eraint_land_ana_'//CRLXDAT1//'_'//CRLXHOU1// &
            & 'H_040.nc'
               
    INCRESULT = NF90_OPEN(CRLN040,NF90_NOWRITE,INCID)
                 
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS OPENING ', CRLN040, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM IN NF90_OPEN')
    ENDIF

    INCRESULT = NF90_INQ_VARID(INCID,NUM_040_NAME,NUM_040_VARID)
        
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR ID', CRLN040, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_INQ_VARID')
    ENDIF

    INCRESULT = NF90_GET_VAR(INCID,NUM_040_VARID,ZRLX, &
              & START=(/1,1/),COUNT=(/NGPTOTG,1/))             

    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR', CRLN040, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_GET_VAR')
    ENDIF

    INCRESULT = NF90_CLOSE(INCID)

    DO INGPT=1,NGPTOTG
       ZRLXM1(INGPT) = ZRLX(INGPT)
       IF (ZRLXM1(INGPT) <= 0.0_JPRB) ZRLXM1(INGPT)=0.0_JPRB 
    ENDDO
    
! READ SOIL MOISTURE LEVEL 2 AT T2

    CRLN040 = TRIM(RLXSMDIR) // &
            & '/eraint_land_ana_'//CRLXDAT2//'_'//CRLXHOU2// &
            & 'H_040.nc'

    INCRESULT = NF90_OPEN(CRLN040,NF90_NOWRITE,INCID)
        
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS OPENING ', CRLN040, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM IN NF90_OPEN')
    ENDIF

    INCRESULT = NF90_INQ_VARID(INCID,NUM_040_NAME,NUM_040_VARID)
        
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR ID', CRLN040, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_INQ_VARID')
    ENDIF

    INCRESULT = NF90_GET_VAR(INCID,NUM_040_VARID,ZRLX, &
              & START=(/1,1/),COUNT=(/NGPTOTG,1/))             

    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR', CRLN040, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_GET_VAR')
    ENDIF

    INCRESULT = NF90_CLOSE(INCID)

    DO INGPT=1,NGPTOTG
       ZRLXP1(INGPT) = ZRLX(INGPT)
       IF (ZRLXP1(INGPT) <= 0.0_JPRB) ZRLXP1(INGPT)=0.0_JPRB 
    ENDDO

    DO INGPT=1,NGPTOTG
       YRLX040(INGPT) & 
            & = (RDIFF21-RDIFFC1)*ZRLXM1(INGPT)/RDIFF21 &
            &    +(RDIFF21-RDIFF2C)*ZRLXP1(INGPT)/RDIFF21
       IF (YRLX040(INGPT) <= 0.0_JPRB) YRLX040(INGPT)=0.0_JPRB 
    ENDDO

! READ SOIL MOISTURE LEVEL 3 AT T1
    
    CRLN041 = TRIM(RLXSMDIR) // &
            & '/eraint_land_ana_'//CRLXDAT1//'_'//CRLXHOU1// &
            & 'H_041.nc'
               
    INCRESULT = NF90_OPEN(CRLN041,NF90_NOWRITE,INCID)
                 
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS OPENING ', CRLN041, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM IN NF90_OPEN')
    ENDIF
     
    INCRESULT = NF90_INQ_VARID(INCID,NUM_041_NAME,NUM_041_VARID)
        
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR ID', CRLN041, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_INQ_VARID')
    ENDIF
     
    INCRESULT = NF90_GET_VAR(INCID,NUM_041_VARID,ZRLX, &
              & START=(/1,1/),COUNT=(/NGPTOTG,1/))             
     
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR', CRLN041, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_GET_VAR')
    ENDIF
     
    INCRESULT = NF90_CLOSE(INCID)
     
    DO INGPT=1,NGPTOTG
       ZRLXM1(INGPT) = ZRLX(INGPT)
       IF (ZRLXM1(INGPT) <= 0.0_JPRB) ZRLXM1(INGPT)=0.0_JPRB 
    ENDDO
     
! READ SOIL MOISTURE LEVEL 3 AT T2

    CRLN041 = TRIM(RLXSMDIR) // &
            & '/eraint_land_ana_'//CRLXDAT2//'_'//CRLXHOU2// &
            & 'H_041.nc'

    INCRESULT = NF90_OPEN(CRLN041,NF90_NOWRITE,INCID)
        
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS OPENING ', CRLN041, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM IN NF90_OPEN')
    ENDIF

    INCRESULT = NF90_INQ_VARID(INCID,NUM_041_NAME,NUM_041_VARID)
        
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR ID', CRLN041, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_INQ_VARID')
    ENDIF

    INCRESULT = NF90_GET_VAR(INCID,NUM_041_VARID,ZRLX, &
              & START=(/1,1/),COUNT=(/NGPTOTG,1/))             

    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR', CRLN041, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_GET_VAR')
    ENDIF

    INCRESULT = NF90_CLOSE(INCID)

    DO INGPT=1,NGPTOTG
       ZRLXP1(INGPT) = ZRLX(INGPT)
       IF (ZRLXP1(INGPT) <= 0.0_JPRB) ZRLXP1(INGPT)=0.0_JPRB 
    ENDDO

    DO INGPT=1,NGPTOTG
       YRLX041(INGPT) & 
            & = (RDIFF21-RDIFFC1)*ZRLXM1(INGPT)/RDIFF21 &
            &    +(RDIFF21-RDIFF2C)*ZRLXP1(INGPT)/RDIFF21
       IF (YRLX041(INGPT) <= 0.0_JPRB) YRLX041(INGPT)=0.0_JPRB 
    ENDDO

! READ SOIL MOISTURE LEVEL 4 AT T1
    
    CRLN042 = TRIM(RLXSMDIR) // &
            & '/eraint_land_ana_'//CRLXDAT1//'_'//CRLXHOU1// &
            & 'H_042.nc'
               
    INCRESULT = NF90_OPEN(CRLN042,NF90_NOWRITE,INCID)
                 
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS OPENING ', CRLN042, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM IN NF90_OPEN')
    ENDIF

    INCRESULT = NF90_INQ_VARID(INCID,NUM_042_NAME,NUM_042_VARID)
        
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR ID', CRLN042, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_INQ_VARID')
    ENDIF

    INCRESULT = NF90_GET_VAR(INCID,NUM_042_VARID,ZRLX, &
              & START=(/1,1/),COUNT=(/NGPTOTG,1/))             

    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR', CRLN042, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_GET_VAR')
    ENDIF

    INCRESULT = NF90_CLOSE(INCID)

    DO INGPT=1,NGPTOTG
       ZRLXM1(INGPT) = ZRLX(INGPT)
       IF (ZRLXM1(INGPT) <= 0.0_JPRB) ZRLXM1(INGPT)=0.0_JPRB 
    ENDDO
     
! READ SOIL MOISTURE LEVEL 4 AT T2

    CRLN042 = TRIM(RLXSMDIR) // &
            & '/eraint_land_ana_'//CRLXDAT2//'_'//CRLXHOU2// &
            & 'H_042.nc'
               
    INCRESULT = NF90_OPEN(CRLN042,NF90_NOWRITE,INCID)
        
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS OPENING ', CRLN042, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM IN NF90_OPEN')
    ENDIF

    INCRESULT = NF90_INQ_VARID(INCID,NUM_042_NAME,NUM_042_VARID)
        
    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR ID', CRLN042, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_INQ_VARID')
    ENDIF

    INCRESULT = NF90_GET_VAR(INCID,NUM_042_VARID,ZRLX, &
              & START=(/1,1/),COUNT=(/NGPTOTG,1/))             

    IF ( INCRESULT /= NF90_NOERR ) THEN
       WRITE(NULERR,*) 'PROBLEMS GETTING VAR', CRLN042, INCRESULT
       CALL ABOR1 ('UDPRLXSM: PROBLEM WITH NF90_GET_VAR')
    ENDIF

    INCRESULT = NF90_CLOSE(INCID)

    DO INGPT=1,NGPTOTG
       ZRLXP1(INGPT) = ZRLX(INGPT)
       IF (ZRLXP1(INGPT) <= 0.0_JPRB) ZRLXP1(INGPT)=0.0_JPRB 
    ENDDO

    DO INGPT=1,NGPTOTG
       YRLX042(INGPT) & 
            & = (RDIFF21-RDIFFC1)*ZRLXM1(INGPT)/RDIFF21 &
            &    +(RDIFF21-RDIFF2C)*ZRLXP1(INGPT)/RDIFF21
       IF (YRLX042(INGPT) <= 0.0_JPRB) YRLX042(INGPT)=0.0_JPRB 
    ENDDO

    DO INGPT=1,NGPTOTG
       ZBUF(INGPT) = YRLX039(INGPT)
    ENDDO
    CALL ISNDGPF(ZBUF,ZFLD039)

    DO INGPT=1,NGPTOTG
       ZBUF(INGPT) = YRLX040(INGPT)
    ENDDO
    CALL ISNDGPF(ZBUF,ZFLD040)

    DO INGPT=1,NGPTOTG
       ZBUF(INGPT) = YRLX041(INGPT)
    ENDDO
    CALL ISNDGPF(ZBUF,ZFLD041)

    DO INGPT=1,NGPTOTG
       ZBUF(INGPT) = YRLX042(INGPT)
    ENDDO
    CALL ISNDGPF(ZBUF,ZFLD042)

  ENDIF MYPROCSMIF

ENDIF LRXSMIF

! ALL PROCESSORS SHOULD NOW RECEIVE

CALL IRCVGPF(ZRLX039,ZFLD039,1)
CALL IRCVGPF(ZRLX040,ZFLD040,1)
CALL IRCVGPF(ZRLX041,ZFLD041,1)
CALL IRCVGPF(ZRLX042,ZFLD042,1)

! Synchronize IDATEREF
ITAG=19591205
CALL MPL_BROADCAST(IDATECH,KTAG=ITAG,KROOT=1,CDSTRING='UPDRLXSM')

WRITE(NULOUT,*) 'UPDRLXSM finished'
CALL FLUSH(NULOUT)

IF (LHOOK) CALL DR_HOOK('UPDRLXSM',1,ZHOOK_HANDLE)
END SUBROUTINE UPDRLXSM

