SUBROUTINE SURLXSM(KULOUT)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEPHY   , ONLY : LECURR   ,LE4ALB   ,LELAIV
USE YOMLUN   , ONLY : NULOUT   ,NULNAM, NULERR
USE YOMMCC   , ONLY : NDIFC    ,NJDCR, &
 & NOACOMM  ,LMCC01   ,LMCC02   ,LMCC03   ,LMCC04   ,&
 & LMCCEC   ,LMCCIEC  ,LMCC12   ,LMCC19   ,LN923    ,LMCCICEIC,&
 & LMCC05   ,NFRCPL   ,NFRCPL_CHE  
USE YOMRLXSM , ONLY : LRXSM, LRXSMT1, LRXSMT2, LRXSMT3, LRXSMT4, LRXSMS, &
 & ZRLX039, ZRLX040, ZRLX041, ZRLX042, RLXSMDIR
USE YOMCT0   , ONLY : NCONF    ,NSTOP    ,NFRCO     ,NFRCO_CHEM ,NPRINTLEV, LALLOPR
USE YOMCT0B  , ONLY : LECMWF
USE YOMDYN   , ONLY : TSTEP
USE YOMGEM   , ONLY : NGPTOT, NGPTOTG
USE YOMDIM   , ONLY : NPROMA

USE YOMMP    , ONLY : MYPROC

USE YOMECEARTH, ONLY : LECEARTH, LECEARTH_UNCOUP

!**** *SURLXSM* * - ROUTINE TO INITIALIZE SOIL MOISTURE NUDGING SWITCHES

!     PURPOSE.
!     --------
!        SET DEFAULT VALUES, THEN READS NAMELIST NAMRLXSM

!**   INTERFACE.
!     ----------
!        *CALL* *SURLXSM(KULOUT)*

!     EXPLICIT ARGUMENTS :  KULOUT
!     --------------------

!     IMPLICIT ARGUMENTS :
!     --------------------
!        COMMON  YOMRLXSM

!     METHOD.
!     -------
!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------

!     AUTHOR.
!     -------
!        Wilhelm May

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : July 2017, based on SUMCC, and corresponding 
!                              modifications by Ida Ringgaard  

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
INTEGER(KIND=JPIM)               :: IOPROC, ISTEP, NITIM
REAL(KIND=JPRB) :: ZHOOK_HANDLE
LOGICAL    :: LLP

#include "inicou.intfb.h"
#include "posnam.intfb.h"
#include "sucom.intfb.h"

#include "namrlxsm.h"
!      ----------------------------------------------------------------

!*       1.    SET DEFAULT VALUES.
!              -------------------

!        1.1 Set implicit default values

IF (LHOOK) CALL DR_HOOK('SURLXSM',0,ZHOOK_HANDLE)

LRXSM=0 
LRXSMT1=0
LRXSMT2=0
LRXSMT3=0
LRXSMT4=0
LRXSMS=1
RLXSMDIR='UNKNOWN-DIR'

!* 
!*       2.    MODIFIES DEFAULT VALUES.
!              ------------------------

CALL POSNAM(NULNAM,'NAMRLXSM')
READ(NULNAM,NAMRLXSM)

! Allocates the necessary space

IF (LRXSM /= 0) THEN
  ALLOCATE(ZRLX039(NGPTOT))
  ALLOCATE(ZRLX040(NGPTOT))
  ALLOCATE(ZRLX041(NGPTOT))
  ALLOCATE(ZRLX042(NGPTOT))
ENDIF

!      -----------------------------------------------------------

!*       3.    PRINTS FINAL VALUES.
!              --------------------

WRITE(UNIT=KULOUT,FMT='('' COMMON YOMRLXSM '')')
WRITE(UNIT=KULOUT,FMT='(&
 & '' LRXSM = '',I3,'' LRXSMT1 = '',I3,'' LRXSMT2 = '',I3, &
 & '' LRXSMT3 = '',I3, '' LRXSMT4 = '',I3, '' LRXSMS = '',I3 )') &
 & LRXSM, LRXSMT1, LRXSMT2, LRXSMT3, LRXSMT4, LRXSMS
 
!      ----------------------------------------------------------------


IF (LHOOK) CALL DR_HOOK('SURLXSM',1,ZHOOK_HANDLE)
END SUBROUTINE SURLXSM
