SUBROUTINE CNT31S
USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMLUN1S , ONLY : NULOUT
USE YOMDPHY  , ONLY : ISLAB,NPDONE,NCSS
USE CPLNG
USE YOMECEARTH , ONLY : LLPJG
IMPLICIT NONE
#ifdef DOC

!**** *CNT31S*  - Controls integration job at level 3

!     Purpose.
!     --------


!**   Interface.
!     ----------
!        *CALL* *CNT31S

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!      Calls SUINIF1S, CNT4
!      Called by CNT2

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the one-column surface IFS

!     Author.
!     -------
!        Jean-Francois Mahfouf and Pedro Viterbo  *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-06-16
!     ------------------------------------------------------------
#endif

#include "suinif1s.intfb.h"
#include "cnt41s.intfb.h"

LOGICAL LOPLEFT		! More gridpoints to do?

LOPLEFT=.TRUE.
ISLAB=1
NPDONE=0
!     ------------------------------------------------------------

!*       1.    Initialize LEVEL 3 COMMONS.
!              ---------------------------

!* Configure OASIS Coupling 
CALL CPLNG_CONFIG
IF (LLPJG) THEN
  ! We initialize the coupling arrays with reasonable values here
  ! The coupling arrays include all grid, while we only run land points
  ! This guarantees that the arrays have some resonable data to avoid error
  CPLNG_FLD(CPLNG_IDX('T2MVeg'))%D(:,1,1) = 280._JPRB
  CPLNG_FLD(CPLNG_IDX('SSRVeg'))%D(:,1,1) = 0._JPRB
  CPLNG_FLD(CPLNG_IDX('SLRVeg'))%D(:,1,1) = 0._JPRB
  CPLNG_FLD(CPLNG_IDX('SoilMVeg'))%D(:,1:NCSS,1) = 0.2_JPRB
  CPLNG_FLD(CPLNG_IDX('SoilTVeg'))%D(:,1:NCSS,1) = 280._JPRB
  CPLNG_FLD(CPLNG_IDX('SDensVeg'))%D(:,1,1) = 330._JPRB
  CPLNG_FLD(CPLNG_IDX('SDVeg'))%D(:,1,1) = 0._JPRB
  CPLNG_FLD(CPLNG_IDX('TPVeg'))%D(:,1,1) = 0._JPRB 
ENDIF 

DO 
  WRITE(UNIT=NULOUT,FMT='(A,I3)')' START CNT31S SLAB: ',ISLAB
  CALL SUINIF1S(LOPLEFT)
!      -----------------------------------------------------------


!*       2.    INTEGRATION.
!              ------------

  CALL CNT41S
!      -----------------------------------------------------------

  WRITE(UNIT=NULOUT,FMT='('' END CNT31S'')')
  IF(.NOT.LOPLEFT)EXIT
  ISLAB=ISLAB+1
ENDDO
!     ------------------------------------------------------------

RETURN
END SUBROUTINE CNT31S
