SUBROUTINE UPDTIM1S(KSTEP,PTDT,PTSTEP)


USE YOMLUN1S , ONLY : NULOUT
USE YOMLOG1S , ONLY : CFFORC
USE YOMRIP   , ONLY : NSSSSS   ,NSTADD   ,NSTASS   ,RTIMST   , &
     &            RSTATI   ,RTIMTR   ,RHGMT    ,REQTIM   ,RSOVR    ,&
     &            RDEASO   ,RDECLI   ,RWSOVR   ,RIP0     ,RCODEC   ,&
     &            RSIDEC   ,RCOVSR   ,RSIVSR   ,RTDT
USE YOMCST   , ONLY : RPI      ,RDAY     ,REA      ,RI0, REPSM
USE YOERIP   , ONLY : RIP0M    ,RCODECM  ,RSIDECM  ,RCOVSRM  ,&
     &            RSIVSRM
USE YOMDYN1S , ONLY : NSTEP    ,TSTEP
USE YOMCC1S  , ONLY : VCALB ,VCLAIL   ,VCLAIH  

USE YOMGPD1S , ONLY : VFALBF   ,&
     &                VFALUVP,VFALUVD,VFALNIP,VFALNID , &
     &                VFLAIL,VFLAIH ,VFTVL,VFTVH,VFCVH,VFCVL,VFPGLOB

USE YOMDPHY  , ONLY : NPOI,NLON,NLAT,NLALO
USE YOMGC1S  , ONLY : LMASK
USE YOEPHY, ONLY: LECTESSEL, LECLIM10D
USE YOMCT01S , ONLY : NSTART
USE YOMCLIE, ONLY: TVEG,UPDFLD,RETIND
USE YOMECEARTH, ONLY: LRDVEG,LLPJG,LLPJGFB
USE MPL_MODULE
USE RTIME2DATE_MOD
USE CPLNG
#ifdef DOC

!**** *UPDTIM1S* - UPDATE TIME OF THE ONE COLUMN SURFACE MODEL 

!     Purpose.
!     --------
!     UPDATE TIME OF THE ONE COLUMN SURFACE MODEL 

!**   Interface.
!     ----------
!        *CALL* *UPDTIM1S(KSTEP,PTDT,PTSTEP)

!        Explicit arguments :
!        --------------------
!        KSTEP : TIME STEP INDEX
!        PTDT  : TIME STEP LEAPFROG
!        PTSTEP: TIME STEP

!        Implicit arguments :
!        --------------------
!        YOMRIP

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the 
!        one column surface model 

!     Author.
!     -------
!        Jean-Francois Mahfouf and Pedro Viterbo  *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-03-22
!        E. Dutra : Sep 2018: Implement LRDVEG
!        E. Dutra : Sep 2018: Exchange with LPJG via OASIS 
!        E. Dutra : Nov 2018: Add JPIB for runs > 69 years

!     ------------------------------------------------------------------
#endif

USE PARKIND1  ,ONLY : JPIM     ,JPRB , JPRD,JPIB

IMPLICIT NONE
#include "fctast.h"
#include "fcttim.h"
#include "minmax.intfb.h"

!* Arguments
INTEGER(KIND=JPIM) :: KSTEP
REAL(KIND=JPRB) :: PTDT,PTSTEP

!* Local variables
INTEGER(KIND=JPIM) :: ITIME,IPR,NRADFR,ISTADD,ISTASS,IYMD,IHM,IDD,&
     &      IMM,IYYYY,IHH,ISS,IMT1,IMT2,IYT1,IYT2,JL,IMT11,IMT12,IDD1,IDD2
REAL(KIND=JPRB) :: ZTETA,ZSTATI,ZHGMT,ZDEASOM,ZDECLIM,ZEQTIMM,ZSOVRM,&
     &      ZWSOVRM,ZJUL,ZTIMTR,ZT1,ZT2,ZT,ZWEI1,ZWEI2
INTEGER(KIND=JPIB) :: IZT  ! Run longer than 69 years

INTEGER(KIND=JPIM) :: JV,IDATE1,IDATE2,ISECO1,ISECO2,IFLD,JLL
REAL(KIND=JPRB)    :: ZPOID1,ZPOID2,ZIWEI1,ZIWEI2

! LPJ-GUESS variables and structures
REAL(KIND=JPRB) :: ZLAIL, ZLAIH, ZCVL, ZCVH, ZTVL, ZTVH, COVERSUM
REAL(KIND=JPRB),POINTER :: VEG_LAIL(:)
REAL(KIND=JPRB),POINTER :: VEG_LAIH(:)
REAL(KIND=JPRB),POINTER :: VEG_CVL(:)
REAL(KIND=JPRB),POINTER :: VEG_CVH(:)
REAL(KIND=JPRB),POINTER :: VEG_TVL(:)
REAL(KIND=JPRB),POINTER :: VEG_TVH(:)
                    
ITIME=NINT(PTSTEP)
IZT=NINT(PTSTEP*(REAL(KSTEP,KIND=JPRD)+0.5),KIND=JPIB)
!CBH      IZT=NINT(PTSTEP*(REAL(KSTEP)))
RSTATI=REAL(IZT,KIND=JPRD)
NSTADD=IZT/NINT(RDAY)
NSTASS=MOD(IZT,NINT(RDAY))
RTIMTR=RTIMST+RSTATI
IPR=0
IF(IPR.EQ.1)THEN
  WRITE(UNIT=NULOUT,FMT='(1X,'' TIME OF THE MODEL '',E20.14,&
     & '' TIME SINCE START '',E20.14)') RTIMTR,RSTATI
ENDIF
RHGMT=REAL(MOD(NINT(RSTATI)+NSSSSS,NINT(RDAY)),KIND=JPRD)

ZTETA=RTETA(RTIMTR)
RDEASO=RRS(ZTETA)
RDECLI=RDS(ZTETA)
REQTIM=RET(ZTETA)
RSOVR =REQTIM+RHGMT
RWSOVR=RSOVR*2.*RPI/RDAY
RIP0=RI0*REA*REA/(RDEASO*RDEASO)

RCODEC=COS(RDECLI)
RSIDEC=SIN(RDECLI)

RCOVSR=COS(RWSOVR)
RSIVSR=SIN(RWSOVR)

RTDT=PTDT


!          2.   PARAMETERS FOR ECMWF-STYLE INTERMITTENT RADIATION 
!               -------------------------------------------------

NRADFR=1
ITIME=NINT( TSTEP)
IZT=NINT( TSTEP*(REAL(KSTEP,KIND=JPRD)+0.5),KIND=JPIB)
!CBH      IZT=NINT( TSTEP*(REAL(KSTEP)))
ZSTATI=REAL(IZT,KIND=JPRD)+0.5*NRADFR*ITIME
ISTADD=IZT/NINT(RDAY)
ISTASS=MOD(IZT,NINT(RDAY))
ZTIMTR=RTIMST+ZSTATI
ZHGMT=REAL(MOD(NINT(ZSTATI)+NSSSSS,NINT(RDAY)),KIND=JPRD)

ZTETA=RTETA(ZTIMTR)
ZDEASOM=RRS(ZTETA)
ZDECLIM=RDS(ZTETA)
ZEQTIMM=RET(ZTETA)
ZSOVRM =ZEQTIMM+ZHGMT                                 
ZWSOVRM=ZSOVRM*2.*RPI/RDAY
RIP0M=RI0*REA*REA/(ZDEASOM*ZDEASOM)

RCODECM=COS(ZDECLIM)
RSIDECM=SIN(ZDECLIM)

RCOVSRM=COS(ZWSOVRM)
RSIVSRM=SIN(ZWSOVRM)


!          2.   MODIFY SEASONALLY VARYING FIELDS 
!               --------------------------------

CALL DATTIM(ZJUL,IYMD,IHM)
IDD=NDD(IYMD)
IMM=NMM(IYMD)
IYYYY=NCCAA(IYMD)
IHH=IHM/100
ISS=60*IHH+60*MOD(IHM,100)

!* This is the old default method using 12 Climate fields 
IF ( NSTEP == NSTART .OR. IHM == 0000) THEN
!  Update albedo at 00 GMT, or define it for the first time step

  IF (IDD >= 15) THEN
    IMT1=IMM
    IMT2=1+MOD(IMM,12)
    IYT1=IYYYY
    IF(IMT2 == 1) THEN
    IYT2=IYT1+1
    ELSE
    IYT2=IYT1
    ENDIF
  ELSE
    IMT1=1+MOD(IMM+10,12)
    IMT2=IMM
    IF(IMT1 == 12) THEN
      IYT1=IYYYY-1
    ELSE
      IYT1=IYYYY
    ENDIF
    IYT2=IYYYY
  ENDIF
  ZT1=RTIME(IYT1,IMT1,15,0)
  ZT2=RTIME(IYT2,IMT2,15,0)

  IMT11=IMT1
  IMT12=IMT2

  ! ZT=RTIME(IYYYY,IMM,IDD,ISS)
  ZT=RTIME(IYYYY,IMM,IDD,0)  !UPDATED ASSUMING WE'RE AT 00UTC 
  ZWEI1=(ZT2-ZT)/(ZT2-ZT1)
  ZWEI2=1.-ZWEI1

  VFALBF(:)=ZWEI1*VCALB(:,IMT11)+ZWEI2*VCALB(:,IMT12)
  VFALUVP(:)=ZWEI1*VCALB(:,IMT11)+ZWEI2*VCALB(:,IMT12)
  VFALUVD(:)=ZWEI1*VCALB(:,IMT11)+ZWEI2*VCALB(:,IMT12)
  VFALNIP(:)=ZWEI1*VCALB(:,IMT11)+ZWEI2*VCALB(:,IMT12)
  VFALNID(:)=ZWEI1*VCALB(:,IMT11)+ZWEI2*VCALB(:,IMT12)
  VFLAIL(:)=ZWEI1*VCLAIL(:,IMT11)+ZWEI2*VCLAIL(:,IMT12)
  VFLAIH(:)=ZWEI1*VCLAIH(:,IMT11)+ZWEI2*VCLAIH(:,IMT12)

ENDIF ! End of Default 

!* If we read vegetation from surfveg and LPJG Feedback is off : 
IF (LRDVEG .AND. .NOT. LLPJGFB) THEN
  IF (NSTEP == NSTART .OR. IHM == 0000) THEN
    !* Update fields only at 00 
    
    ZT=RTIME(IYYYY,IMM,IDD,ISS)  ! current time of the model 
    CALL UPDFLD(ZT,TVEG,2) ! update indices and data 
    ! Compute Linear interpolation coef. 
    ZWEI1=(TVEG%FTIME(TVEG%ISTEP(2))-ZT)/&
           &(TVEG%FTIME(TVEG%ISTEP(2))-TVEG%FTIME(TVEG%ISTEP(1)))
    ZWEI2=1._JPRB-ZWEI1
    ZIWEI1=NINT(ZWEI1)
    ZIWEI2=1._JPRB - ZIWEI1
    
    !* Debug info to output 
    WRITE(UNIT=NULOUT,FMT='(1X,'' UPDTIM1S:Interpolating STEP1:'',I5,'' STEP2:'',I5,&
    & '' ZWEI1:'',F5.3,'' ZWEI2:'',F5.3,'' ZIWEI1:'',F3.1,'' ZIWEI2:'',F3.1)') &
    &TVEG%ISTEP(1),TVEG%ISTEP(2),ZWEI1,ZWEI2,ZIWEI1,ZIWEI2
    CALL RTIME2DATE(TVEG%FTIME(TVEG%ISTEP(1)),IDATE1,ISECO1)
    CALL RTIME2DATE(TVEG%FTIME(TVEG%ISTEP(2)),IDATE2,ISECO2)
    WRITE(UNIT=NULOUT,FMT='(1X,''STEP1 Date:'',I9,1x,I6,'' STEP2 Date:'',I9,1x,I6)')&
    & IDATE1,ISECO1,IDATE2,ISECO2
    
    !* Actual calculations
    IFLD = RETIND('cvl',TVEG%IFIELDS)
    VFCVL= TVEG%DATA(1,:,IFLD)*ZWEI1+TVEG%DATA(2,:,IFLD)*ZWEI2
    
    IFLD = RETIND('cvh',TVEG%IFIELDS)
    VFCVH= TVEG%DATA(1,:,IFLD)*ZWEI1+TVEG%DATA(2,:,IFLD)*ZWEI2
    
    IFLD = RETIND('tvl',TVEG%IFIELDS)
    VFTVL= TVEG%DATA(1,:,IFLD)*ZIWEI1+TVEG%DATA(2,:,IFLD)*ZIWEI2
    
    IFLD = RETIND('tvh',TVEG%IFIELDS)
    VFTVH= TVEG%DATA(1,:,IFLD)*ZIWEI1+TVEG%DATA(2,:,IFLD)*ZIWEI2
    
    IFLD = RETIND('lai_lv',TVEG%IFIELDS)
    VFLAIL= TVEG%DATA(1,:,IFLD)*ZWEI1+TVEG%DATA(2,:,IFLD)*ZWEI2
    
    IFLD = RETIND('lai_hv',TVEG%IFIELDS)
    VFLAIH= TVEG%DATA(1,:,IFLD)*ZWEI1+TVEG%DATA(2,:,IFLD)*ZWEI2   
    
    !* If coupled with LPJ VFCVL,VFCVH,VFTVL,VFTVH,VFLAIL,VFLAIH 
    ! * Should come from OASIS ... 
    
    
  ENDIF !Update fields
ENDIF !LRDVEG

!* If LPJG is coupled we receive the fields 
IF (LLPJG) THEN
   CALL CPLNG_EXCHANGE(CPLNG_STAGE_RCV_VEG)
   VEG_LAIL => CPLNG_FLD(CPLNG_IDX('LAILVeg'))%D(:,1,1)
   VEG_LAIH => CPLNG_FLD(CPLNG_IDX('LAIHVeg'))%D(:,1,1)
   VEG_CVL => CPLNG_FLD(CPLNG_IDX('FracLVeg'))%D(:,1,1)
   VEG_CVH => CPLNG_FLD(CPLNG_IDX('FracHVeg'))%D(:,1,1)
   VEG_TVL => CPLNG_FLD(CPLNG_IDX('TypeLVeg'))%D(:,1,1)
   VEG_TVH => CPLNG_FLD(CPLNG_IDX('TypeHVeg'))%D(:,1,1)
ENDIF

!* If LPJG is coupled and Feedback is on we use LPJG Fields 
IF (LLPJG .AND. LLPJGFB) THEN
  IF (NSTEP == NSTART .OR. IHM == 0000) THEN
    ! Print some diagnostics 
    WRITE(NULOUT,'(A)') "Updating OSM with LPJG FIELDS in UPDTIM1S"
    CALL MINMAX('LPJ:LAIL',VEG_LAIL,NLALO,1,LMASK,NULOUT)
    CALL MINMAX('LPJ:LAIH',VEG_LAIH,NLALO,1,LMASK,NULOUT)
    CALL MINMAX('LPJ:CVL',VEG_CVL,NLALO,1,LMASK,NULOUT)
    CALL MINMAX('LPJ:CVH',VEG_CVH,NLALO,1,LMASK,NULOUT)
    CALL MINMAX('LPJ:TVL',VEG_TVH,NLALO,1,LMASK,NULOUT)
    CALL MINMAX('LPJ:TVH',VEG_LAIL,NLALO,1,LMASK,NULOUT)
    ! We must loop on land points only as OASIS Buffer contains all the domaing
    DO JL=1,NPOI
      JLL=VFPGLOB(JL) ! Global index in OASIS

      ! LOW LAI
      ZLAIL=MAX(0.0_JPRB,VEG_LAIL(JLL))
      VFLAIL(JL)=ZLAIL

      ! HIGH LAI
      ZLAIH=MAX(0.0_JPRB,VEG_LAIH(JLL))
      VFLAIH(JL)=ZLAIH

      ! LOW COVER FRACTION
      ZCVL=MAX(0.0_JPRB,MIN(1.0_JPRB,VEG_CVL(JLL)))

      ! HIGH COVER FRACTION
      ZCVH=MAX(0.0_JPRB,MIN(1.0_JPRB,VEG_CVH(JLL)))

      COVERSUM = ZCVL + ZCVH
      ! Rescale if necessary to catch rounding errors leading to cover
      ! fractions > 1
      IF (COVERSUM > 1._JPRB) THEN
          WRITE(UNIT=NULOUT,FMT='(A,I6,I6)') 'LPJG - RESCALING LPJ-GUESS FIELDS ',JL,JLL
          ZCVL = ZCVL / COVERSUM
          ZCVH = ZCVH / COVERSUM
      ENDIF

      ! UPDATE COVER FRACTIONS
      VFCVL(JL)=ZCVL
      VFCVH(JL)=ZCVH

      ! LOW COVER TYPE. Restrict to be between 0 and 20
      ZTVL=VEG_TVL(JLL)
      IF (ZTVL==8.0_JPRB) THEN
          ZTVL=0.0_JPRB ! Force deserts to have ZTVL=0
      ENDIF
      VFTVL(JL)=MIN(MAX(ZTVL,0.0_JPRB),20.0_JPRB)

      ! HIGH COVER TYPE. Restrict to be between 0 and 20
      ZTVH=VEG_TVH(JLL)
      VFTVH(JL)=MIN(MAX(ZTVH,0.0_JPRB),20.0_JPRB)   
    ENDDO
  ENDIF

ENDIF ! LLPGFB

! STOP
RETURN
END SUBROUTINE UPDTIM1S
