#!/usr/bin/env python

import argparse
from gribapi import *
import numpy as np

#-----------------------------------------------------------------------
# get grid information from lsm
def get_grib_info(fin,verbose):
  if verbose:
    print 'get_grib_info: reading file',fin
  f = open(fin)
  gid = grib_new_from_file(f)
  npp = grib_get_size(gid,'values')
  nlat = grib_get(gid,'numberOfPointsAlongAMeridian')
  nplon = np.empty(nlat)
  nplon = grib_get_array(gid,'pl')
  missingValue = grib_get_double(gid,"missingValue")
  # get lat,lon,lsm
  iterid = grib_iterator_new(gid,0)
  lat = np.empty(npp)
  lon = np.empty(npp)
  values = np.empty(npp)
  for ip in xrange(0, npp, 1):
    [  lat[ip], lon[ip], values[ip]] = grib_iterator_next(iterid)
  grib_iterator_delete(iterid)
  if verbose:
    print 'get_grib_info: np,nlat=',npp,nlat
    print 'get_grib_info: missing value=',missingValue
    print 'get_grib_info: nplon',nplon[[0 , int(nlat/2) , nlat-1 ]]
    print 'get_grib_info: ip, lat, lon, lsm'


    print 'get_grib_info: land points:',values[np.nonzero(values>0.5)].shape[0]

    for ip in xrange(0, npp, int(round(npp/10))):
      print ip ,lat[ip],lon[ip],values[ip]

  grib_release(gid)
  f.close()
  return npp, nlat, nplon, lat, lon, values
  
def create_latlon_bounds(npp,nlat,nplon,lat,lon,verbose):
  if verbose: print 'create_latlon_bounds:'
  xBounds = np.empty((npp,4))
  yBounds = np.empty((npp,4))

  ipS = 0 # 1 longitude
  for il in xrange(0, nlat ):
    ipE = ipS + nplon[il]-1 # last longitude
    if il == 0:
      lat0 = 90. # upper 
    else:
      lat0 = 0.5*(lat[ipS-1]+lat[ipS])
    if il == nlat-1:
      lat1 = -90. # lower
    else:
      lat1 = 0.5*(lat[ipE+1]+lat[ipE])
    dlon=180./nplon[il] # delta in longitude
    xBounds[ipS:ipE+1,0]=lon[ipS:ipE+1]-dlon
    xBounds[ipS:ipE+1,3]=xBounds[ipS:ipE+1,0]
    xBounds[ipS:ipE+1,1]=lon[ipS:ipE+1]+dlon
    xBounds[ipS:ipE+1,2]=xBounds[ipS:ipE+1,1]
    yBounds[ipS:ipE+1,[0,1]]=lat1
    yBounds[ipS:ipE+1,[2,3]]=lat0
    
    if verbose:
      print il, ipS, ipE,nplon[il], lat[[ipS, ipE]] , lon[[ipS, ipE]]
      print il, lat0, lat[ipS],lat1,dlon
      print xBounds[ipS,:],xBounds[ipE,:]
      print yBounds[ipS,:],yBounds[ipE,:]
      print ' '
    ipS = ipE + 1 # next Start longitude
  
  return xBounds,yBounds

def write_gdes(file_out,npp,nx,ny,xval,yval,xBounds,yBounds,verbose):
  if verbose: print 'write_gdes:',file_out
  f = open(file_out,'w',0)
  f.write("gridtype = curvilinear \n")
  f.write("gridsize = %i \n" % (npp) )
  f.write("xsize    = %i \n" % (nx) )
  f.write("ysize    = %i \n" % (ny) )
  f.write("xvals    = \n"  )
  for v in xval:
    f.write("%.3f \n" % v )
  f.write("yvals    = \n"  )
  for v in yval:
    f.write("%.3f \n" % v )
  f.write("xbounds    = \n"  )
  for il in xrange(0, npp ):
    for ib in xrange(0,4):
      f.write("%.3f " % ( float(xBounds[il,ib])))
    f.write("\n")
  f.write("ybounds    = \n"  )
  for il in xrange(0, npp ):
    for ib in xrange(0,4):
      f.write("%.3f " % ( float(yBounds[il,ib])))
    f.write("\n")
  
  
  f.close()
  #print "gridtype = curvilinear"
#print "gridsize = ", nx * ny
#print "xsize    = ", nx
#print "ysize    = ", ny
#print ""
#print "xvals ="

#--------------------------------------------------------------------------
# get arguments from call 
parser = argparse.ArgumentParser(description='Create cdo grid description file for gaussian reduced grids ".txt" ')
parser.add_argument('-i',dest='file_in',default='input.lsm',type=str,metavar='file_in',
                    help='input grib file to read grid info: rcp c1a:/home/rd/rdx/data/36/climate/${resol}l_2/lsmoro . ')
parser.add_argument('-o',dest='file_out',default='gdes.txt',type=str,metavar='file_out',
                    help='output grid description file in cdo format')
parser.add_argument('-v',action='store_true',dest='verbose',
                    help='verbose')
args = parser.parse_args()
verbose = args.verbose

#---------------------------------------------------------------------------
# read information from file 
[npp, nlat, nplon, lat, lon, lsm] = get_grib_info(args.file_in,verbose)

#---------------------------------------------------------------------------
# create lat/lon bounds 
[xBounds, yBounds] = create_latlon_bounds(npp,nlat,nplon,lat,lon,verbose)

#--------------------------------------------------------------------------
# write to file
write_gdes(args.file_out,npp,npp,1,lon,lat,xBounds,yBounds,verbose)


