#!/usr/bin/env bash

# script to generate the forcing for the OSM
# E. Dutra, October 2018
# P. Miller, July 2019 - added NSC/Tetralith paths
 
function get_greduced()
{
  case $1 in
    T159L* ) greduced=80;;
    T255L* ) greduced=128;;
         * ) greduced=None ;;
  esac
  echo $greduced
}

function get_forcing_gswp3()
{
  fy=$1
  fy1=$(( $fy + 1 ))
  fyy1=$fy1
  [[ $fy = 2014 ]] && fy1=2014
  
  req_grid=$(get_greduced $ifs_grid)
  
#   cp $forcing_raw_dir_gswp3/GSWP3.BC.*.3hrMap.${fy}.nc . 
  
  fgrid=${osm_aux_dir}/lsm_N${req_grid}.grb
  fremap=${osm_aux_dir}/info_cdo_N${req_grid}.nc
  for cvar in ${osm_forcing_vars}
  do
    remapf=remapbil
    [[ $cvar =~ LWdown|SWdown|Rainf|Snowf ]] && remapf=remapcon
    case $cvar in 
      LWdown) cvar1=strd;code=175;;
      PSurf)  cvar1=sp;code=134;;
      Qair) cvar1=q;code=133;;
      Rainf) cvar1=cp;code=143;;
      SWdown) cvar1=ssrd;code=169;;
      Snowf) cvar1=sf;code=144;;
      Tair) cvar1=t;code=130;;
      Wind) cvar1=u;code=131;;
    esac
#     cdo -b P24 -f grb -setgrid,$fgrid -$remapf,$fremap GSWP3.BC.${cvar}.3hrMap.${fy}.nc data.${cvar1}.grb 
    cdo -b P24 -f grb -setgrid,$fgrid -$remapf,$fremap $forcing_raw_dir_gswp3/GSWP3.BC.${cvar}.3hrMap.${fy}.nc data.${cvar1}.grb 
    
    cdo -v -seltimestep,1 $forcing_raw_dir_gswp3/GSWP3.BC.${cvar}.3hrMap.${fy1}.nc tmp3.nc
    cdo -v -setyear,$fyy1 tmp3.nc tmp.nc 
    cdo -b P24 -f grb -setgrid,$fgrid -$remapf,$fremap tmp.nc tmp.grb 
    cat tmp.grb >> data.${cvar1}.grb
    rm -f tmp.nc tmp.grb tmp3.nc
    
    grib_set -s table2Version=128,indicatorOfParameter=$code  data.${cvar1}.grb data.${cvar1}.grb_ ; mv data.${cvar1}.grb_ data.${cvar1}.grb
  done
 
  # Wind gets special treatment - GSWP3 speed goes into to the u-component, and the v-component is set to zero
  cdo mulc,0 data.u.grb data.v.grb
  grib_set -s table2Version=128,indicatorOfParameter=132 data.v.grb data.v.grb_ ; mv data.v.grb_ data.v.grb

}



function get_forcing_era20c()
{
  # generic function to get data 
  ori_grid=80
  fy=$1
  req_grid=$(get_greduced $ifs_grid)
  
  $forcing_raw_subdir && fd=$forcing_raw_dir/e20c || fd=$forcing_raw_dir
  ${copy_cmd} $fd/era20_${fy}.tar.gz . 
  tar -zxvf era20_${fy}.tar.gz
  rm -f era20_${fy}.tar.gz
  
  
  if [[ $req_grid == $ori_grid ]];then
    return
  else
    echo "Requires interpolation"
    for pp in 10u 10v 2d 2t cp lsp sf sp ssrd strd 
    do
      ( 
      ${emos_tool} --reduced=$req_grid data.${pp}.grb data.${pp}.grb.tmp 1>data.${pp}.grb.tmp.out
      mv data.${pp}.grb.tmp data.${pp}.grb
      rm -f data.${pp}.grb.tmp.out
      ) &
    done
    wait
  fi 
}

function get_forcing_erax()
{
  # generic function to get data 
  fy=$1 #forcing year
  ori_grid=$2 #original grid e.g. 320, 128
  subdir=$3 #forcing file subdir
  prefix=$4 #forcing file prefix
  freq=$(printf "%02d" $5) #forcing frequency in hours
  req_grid=$(get_greduced $ifs_grid)
  
  for im in {1..12}
  do
    mm=$(printf "%02d" $im)

    $forcing_raw_subdir && fd=$forcing_raw_dir/$subdir || fd=$forcing_raw_dir
    ${copy_cmd} $fd/${prefix}${fy}${mm}00.tar.gz forcing.tar.gz
    tar -zxvf forcing.tar.gz
    rm -f forcing.tar.gz
    if [[ $im = 1 ]];then
      mv Tair.grb data.t.grb
      mv Qair.grb data.q.grb 
      cdo exp PSurf.grb data.sp.grb
      mv Wind_E.grb data.u.grb 
      mv Wind_N.grb data.v.grb 
      mv SWdown.grb data.ssrd.grb 
      mv LWdown.grb data.strd.grb 
      mv Rainf.grb data.cp.grb 
      mv Snowf.grb data.sf.grb 
    else
      ddateN=$(date -u -d "${fy}-${mm}-01 +1 month" +"%Y-%m-01")
      ddate="${fy}-${mm}-01T${freq}:00:00,${ddateN}T00:00:00"
      cdo seldate,$ddate Tair.grb tmp.grb ; cat tmp.grb >> data.t.grb 
      cdo seldate,$ddate Qair.grb tmp.grb ; cat tmp.grb >> data.q.grb 
      cdo seldate,$ddate -exp PSurf.grb tmp.grb ; cat tmp.grb >> data.sp.grb 
      cdo seldate,$ddate Wind_E.grb tmp.grb ; cat tmp.grb >> data.u.grb 
      cdo seldate,$ddate Wind_N.grb tmp.grb ; cat tmp.grb >> data.v.grb 
      cdo seldate,$ddate SWdown.grb tmp.grb ; cat tmp.grb >> data.ssrd.grb 
      cdo seldate,$ddate LWdown.grb tmp.grb ; cat tmp.grb >> data.strd.grb 
      cdo seldate,$ddate Rainf.grb tmp.grb ; cat tmp.grb >> data.cp.grb 
      cdo seldate,$ddate Snowf.grb tmp.grb ; cat tmp.grb >> data.sf.grb 
    fi
    rm -f Tair.grb Qair.grb PSurf.grb Wind_E.grb Wind_N.grb SWdown.grb LWdown.grb Rainf.grb Snowf.grb Ctpf.grb 

  done

  grib_set -s shortName=sp,gridType=reduced_gg data.sp.grb tmp.grb
  mv -f tmp.grb data.sp.grb
  
  if [[ $req_grid != $ori_grid ]];then
    echo "Requires interpolation"
    for pp in u v q t cp sf sp ssrd strd
    do
      (
      ${emos_tool} --reduced=$req_grid data.${pp}.grb data.${pp}.grb.tmp 1>data.${pp}.grb.tmp.out
      mv data.${pp}.grb.tmp data.${pp}.grb
      rm -f data.${pp}.grb.tmp.out
      ) &
    done
    wait
  fi 
}

set -euxv

osm_forcing_type=$1 #'erai' # 'e20c' # 'gswp3'
year=$2 #1979
ifs_grid=$3 #T255L91 #T159L62

ifs_version=36r4  # static 


##==================================
# 0.1 Variables definition : platform dependent, user must define the platform variable
# TODO get this from ecconf

platform="MN4"

# LISBON
if [[ $platform == "LISBON" ]] ; then
work_dir='/media/tor_disk1/data/emanuel/ECEARTH-RUNS/gen_forcing'  # base work directory
save_dir='/media/tor_disk1/data/emanuel/ECEARTH-RUNS/OSM_FORCING' # base directory to save forcing 
forcing_raw_dir='/media/tor_md3/emanuel/OSM_FORCING'               # base directory to dave forcing
forcing_raw_subdir=false
forcing_raw_dir_gswp3=${forcing_raw_dir}/gswp3
osm_aux_dir="/media/tor_md3/emanuel/ecearth/osm"                   # directory with auxiliary data 
#this has to be set from the calling script
#ecearth_src_dir='/home/emanuel/work/ec-earth/r5829-osm/sources' # source of ec-earth svn repository 
emos_tool="/share/apps/progs/gfortran_4.8.5/libemos_v4.5.4/bin/emos_tool"
copy_cmd="cp "
set +x
module load precompiled/anaconda2/v5.1.0
module load gfortran_4.8.5/cdo/v1.9.3
set -x


# CCA
elif [[ $platform == "CCA" ]] ; then
work_dir="$SCRATCH/ECEARTH-RUNS/gen_forcing"  # base work directory
save_dir="$SCRATCH/OSM_FORCING" # base directory to save forcing 
forcing_raw_dir='ec:/ptb/OSM_FORCING'               # base directory to get forcing
forcing_raw_subdir=false
forcing_raw_dir_gswp3=$SCRATCH/OSM_FORCING/gswp3/raw
osm_aux_dir="/perm/ms/nl/nm6/ECE3-DATA/osm/"                   # directory with auxiliary data 
#this has to be set from the calling script
#ecearth_src_dir='/perm/ms/pt/ptb/ec-earth/r5829-osm/sources' # source of ec-earth svn repository 
emos_tool="/usr/local/apps/libemos/000455/GNU/63/bin/emos_tool"
copy_cmd="ecp "
set +x
module unload PrgEnv-cray
module unload PrgEnv-intel
module unload PrgEnv-gnu
module load PrgEnv-gnu
module load emos
module rm grib_api
module load eccodes
unset GRIB_DEFINITION_PATH
unset GRIB_SAMPLES_PATH
unset GRIB_BIN_PATH
module load python
module load cdo
module load nco
set -x

# MN4
elif [[ $platform == "MN4" ]] ; then
work_dir="$SCRATCH/ECEARTH-RUNS/gen_forcing"  # base work directory
#save_dir="$SCRATCH/ECEARTH-RUNS/OSM_FORCING" # base directory to save forcing 
save_dir="/gpfs/projects/bsc32/bsc32051/OSM_FORCING" # base directory to save forcing 
forcing_raw_dir='/gpfs/projects/bsc32/bsc32051/OSM_FORCING/raw'               # base directory to get forcing
forcing_raw_subdir=true
forcing_raw_dir_gswp3=${forcing_raw_dir}/gswp3
osm_aux_dir="/gpfs/projects/bsc32/models/ecearth/v3.3.0/inidata/osm"                   # directory with auxiliary data 
#this has to be set from the calling script
#ecearth_src_dir='/gpfs/scratch/pr1egi00/pr1egi24/a1bc/auto-ecearth3/sources/sources/' # source of ec-earth svn repository 
emos_tool="/apps/EMOSLIB/4.5.7/INTEL/bin/emos_tool"
copy_cmd="cp "
set +x
module load intel/2018.3 impi/2018.3 mkl/2018.3
module load netcdf/4.2 hdf5/1.8.19 CDO/1.8.2 
module load udunits/2.2.25 gsl/2.4
module load nco/4.2.3_netcdf-4.2
module load eccodes/2.8.0 python/2.7.13
module load fftw emoslib/4.5.7
unset GRIB_DEFINITION_PATH
unset GRIB_SAMPLES_PATH
unset GRIB_BIN_PATH
set -x

# NSC/Tetralith
elif [[ $platform == "NSC" ]] ; then
work_dir="/proj/s-cmip/users/lu/lpjg/gswp3"  # base work directory
save_dir="/proj/s-cmip/users/lu/lpjg/gswp3/osm_forcing" # base directory to save forcing
forcing_raw_dir='/proj/s-cmip/users/lu/lpjg/gswp3'               # base directory to get forcing
forcing_raw_subdir=false
osm_aux_dir="/proj/s-cmip/users/lu/lpjg/gswp3/osm"                   # directory with auxiliary data
ecearth_src_dir='/home/sm_paumi/esm/osm/sources' # source of ec-earth svn repository
#emos_tool="/apps/EMOSLIB/4.5.7/INTEL/bin/emos_tool"
copy_cmd="cp "
set +x

module load eccodes/2.10.0-nsc1-ENABLE_AEC-intel-2018a-eb
# NB! On Tetralith you must activate Python3 with netCDF4 and ecccodes-python loaded, and module load eccodes

set -x


else
  echo "ERROR in generate_forcing.sh you must define platform"
  exit 1
fi

##=====================================
# 0.2 - Generated variables 
osm_forcing_dir=${save_dir}/${osm_forcing_type}/${ifs_grid} # output directory /year


osm_forcing_vars="Tair Qair PSurf Wind SWdown LWdown Snowf Rainf"
osm_forcing_files=""
for cvar in ${osm_forcing_vars} ; do osm_forcing_files+=${cvar}".nc " ; done

##=====================================
# 1. Start work

## check if files alreay exist:
ret=0
for ff in ${osm_forcing_files}
do
  if [[ ! -r ${osm_forcing_dir}/$year/$ff ]]; then
    ret=1
  fi
done
if [[ $ret == 0 ]]; then
  echo "Files already present, skipping year $year"
  exit 0
fi

# go to working directory
WDIR=${work_dir}/${osm_forcing_type}/${ifs_grid}/${year}
mkdir -p $WDIR
cd $WDIR
rm -rfv *

# check year and to see if it is between 1850 and 1900, offset represents the year to copy data from
if [ $year -ge 1850 ] && [ $year -le 1868 ] ; then
  offset=52
elif [ $year -ge 1869 ] && [ $year -le 1888 ] ; then
  offset=32
elif [ $year -ge 1889 ] && [ $year -le 1900 ] ; then
  offset=12
elif [ $year -lt 1850 ] ; then
  echo "error! year must be greater than 1850!"
  exit 1
else
  offset=0
fi

mkdir -p ${osm_forcing_dir}/${year}

# copy data from other years if an offset is required
if [ $offset -ne 0 ] ; then

  end_year=$((year+offset))
  ORGDIR=${osm_forcing_dir}/${end_year}

  echo "Copying from $ORGDIR to ${osm_forcing_dir}/${year}"

  [[ $osm_forcing_type = era5 ]] && taxisfreq=1hour || taxisfreq=3hour
  [[ $osm_forcing_type = e20c ]] && taxis="$(( year-1 ))-12-31,09:00:00,${taxisfreq}" || taxis="${year}-01-01,00:00:00,${taxisfreq}"
  [[ $osm_forcing_type = e20c ]] && tunits="hours since $(( year-1 ))-12-31 09:00:00" || tunits="hours since ${year}-01-01 00:00:00"

  for ff in ${osm_forcing_files}
  do
    cdo -f nc4 -z zip_2 settaxis,"${taxis}" ${ORGDIR}/${ff} ${ff}
    ncatted -a units,time,o,c,"${tunits}" ${ff}
    mv $ff ${osm_forcing_dir}/${year}
  done

else

# get data
case $osm_forcing_type in
  e20c ) get_forcing_era20c $year
         ftype='SFC';Rainf_pp=True;Flx_acc=True;;
  erai ) get_forcing_erax $year 128 erai forcing_ei_1_oper_3_ 3 3
         ftype='ML';Rainf_pp=False;Flx_acc=False;;
  era5 ) get_forcing_erax $year 320 era5 forcing_ea_1_oper_1_ 1 1
         ftype='ML';Rainf_pp=False;Flx_acc=False;;
  gswp3 ) get_forcing_gswp3 $year
         ftype='ML';Rainf_pp=False;Flx_acc=False;;
  *)     echo "osm_forcing_type $osm_forcing_type not coded!"
         exit 1;;
esac

# convert to netcdf
ln -sf ${osm_aux_dir}/surfclim_${ifs_grid} surfclim
cp ${ecearth_src_dir}/ifs-${ifs_version}/src/surf/offline/scripts/create_forcing.py .
cat > pyrun.py << EOF
from create_forcing import *
SURFCLIM="surfclim"
INPUTGRB="data.[shortName].grb"
tunits="seconds since $year-01-01 00:00:00"
process(SURFCLIM,INPUTGRB,'$ftype',tunits,$Rainf_pp,Flx_acc=$Flx_acc)
EOF
python -u pyrun.py

# fix relative humidity - only required for GSWP3
if [[ $osm_forcing_type == "gswp3" ]] ; then
  cp ${ecearth_src_dir}/ifs-${ifs_version}/src/surf/offline/scripts/fix_forcing_relative_humidity.py .
  cp Qair.nc ${osm_forcing_dir}/$year/Qair_orig.nc
  python fix_forcing_relative_humidity.py -iQ Qair.nc -iT Tair.nc -iP PSurf.nc
fi

mv ${osm_forcing_files} ${osm_forcing_dir}/$year

fi #if [ $offset -ne 0 ]

rm -rfv $WDIR
