#!/usr/bin/env bash 

## script to generate submission script 
## to generate forcing 
## E. Dutra October 2018

set -eux 

# user must define the platform variable
platform="MN4"

#if [[ $platform == "LISBON" ]] || ; then
#  sloc=/perm/ms/pt/ptb/ec-earth/r5829-osm/sources/ifs-36r4/src/surf/offline/scripts
#  floc=$sloc
#elif [[ $platform == "CCA" ]] ; then
#  sloc=/perm/ms/pt/ptb/ec-earth/r5829-osm/sources/ifs-36r4/src/surf/offline/scripts
#  floc=$sloc
#elif [[ $platform == "NSC" ]] ; then
#  sloc=/proj/s-cmip/users/lu/lpjg/gswp3
#  floc=$sloc
#elif [[ $platform == "MN4" ]] ; then
#  sloc=`dirname "$0"`
#  if [[ $sloc == "." ]] ; then sloc=`pwd` ; fi
#  floc=`pwd`
#else
#  echo "ERROR in generate_job.sh you must define platform"
#  exit 1
#fi

sloc=`dirname "$0"`
if [[ $sloc == "." ]] ; then sloc=`pwd` ; fi
floc=`pwd`

ftype=$1 #erai
fresol=$2 #T255L91 
ystart=$3 #1979
yend=$4  #2017

tag=gen_forcing_${ftype}_${fresol}_${ystart}_${yend}

if [[ $platform == "LISBON" ]] ; then
cat > ${floc}/${tag}.cmd << EOF
#!/usr/bin/env bash 

#SBATCH -J $tag
#SBATCH -N 1 
#SBATCH -p np16 
#SBATCH -o ${floc}/${tag}_%j.out

set -eux 

EOF

elif [[ $platform == "CCA" ]] ; then
cat > ${floc}/${tag}.cmd << EOF
#!/usr/bin/env bash 

#PBS -N $tag
#PBS -q ns
# #PBS -l EC_billing_account=spptdutr
#PBS -l EC_total_tasks=1
#PBS -l EC_threads_per_task=1
#PBS -l EC_hyperthreads=1
#PBS -j oe
#PBS -l walltime=48:00:00

set -eux 

EOF

elif [[ $platform == "MN4" ]] ; then
cat > ${floc}/${tag}.cmd << EOF
#!/usr/bin/env bash 

#SBATCH -J $tag
#SBATCH -N 1 
#SBATCH -o ${floc}/${tag}_%j.out
#SBATCH -e ${floc}/${tag}_%j.err

##SBATCH -t 12:00:00
#SBATCH -t 02:00:00
#SBATCH --exclusive
# #SBATCH -q debug
# #SBATCH -p interactive

set -eux 

EOF

elif [[ $platform == "NSC" ]] ; then
cat > ${floc}/${tag}.cmd << EOF
#!/bin/sh

EOF

fi

cat >> ${floc}/${tag}.cmd << EOF

export ecearth_src_dir=${sloc}/../../../../../

cd $floc
for y in {${ystart}..${yend}}
# generate forcing files in reverse order so the files for 1850-1899 can be generated after the ones for 1901-2010 are available
#for ((y=$yend; y>=$ystart; y--))
do
  $sloc/generate_forcing.sh $ftype \$y $fresol
done

exit 0
EOF

cd $floc

if [[ $platform == "LISBON" ]] ; then
    sbatch ${floc}/${tag}.cmd
elif [[ $platform == "CCA" ]] ; then
  echo "script generated, launch it with the following command:"
  echo qsub ${floc}/${tag}.cmd
elif [[ $platform == "MN4" ]] ; then
  echo "script generated, launch it with the following command:"
  echo sbatch ${floc}/${tag}.cmd
elif [[ $platform == "NSC" ]] ; then
    # Test with : sbatch -N 1 --account=snic2018-2-11 --reservation=devel -t 20 ${floc}/${tag}
    sbatch -N 1 --account=snic2018-2-11 -t 360 ${floc}/${tag}.cmd
fi


