#!/bin/bash

set -ue

# function copied from auto-ecearth plugins/localtrans.sh

#######################################
# Local transfer of model output 
# Globals:
#   expid, s, m, arch, remove, file_to_copy, how_many_retry_copy_file
# Arguments:
#   1: is the destination path for the output
#   2: is the name of the final folder of the destination path /
#   3: list of files to copy
# Returns:
#   None
#######################################
function transfer_cca() {
  set +xuve
  module load ECAC/4.0.2
  set -xuve
  how_many_retry_copy_file=5
  remote=FALSE
  source_pattern=$1
  destination_path=$2

  source_path=$( dirname $source_pattern[0] )

  set +e
  list_of_files_to_copy=$( ecaccess-file-dir $source_pattern )
  set -e

  if [[ ${list_of_files_to_copy} == "No such file or directory" ]]; then
     list_of_files_to_copy=" "
  fi
  if [[ ${list_of_files_to_copy} == "Permission denied" ]]; then
     echo error ecaccess-file-dir "Permission denied"
     exit 1
  fi

  if [[ ${list_of_files_to_copy} == "ECFS not available (sleep)" ]]; then
      echo error ecaccess-file-dir "ECFS not available (sleep)"
      exit 1
  fi



  #apply mkdir + chmod to handle folder rights properly
  #mkdir_esarchive ${destination_path}

  for file in $list_of_files_to_copy; do
    retry=1
    while [[ $retry -le $how_many_retry_copy_file ]]; do

        echo "transfering file ... " $source_path/$file $destination_path/$file retry $retry

        #ecaccess-file-get $source_path/$file $destination_path/$file
        ecaccess-file-get -progress $source_path/$file $destination_path/$file
        remote_size=$(ecaccess-file-size $source_path/$file)
        local_size=$(ls -lrt $destination_path/$file | awk '{print $5}')

        if [[ "$local_size" == "$remote_size" ]]; then
            #if [[ $remove == TRUE ]]; then
            #   ecaccess-file-delete -force $source_path/$file
            #fi
            echo "done."
            echo
            echo
            break
        else
           echo "Transfer error: Source and destination files have not the same size. Retrying..."
           ((retry++))
        fi
    done

    if [[ $retry -ge $how_many_retry_copy_file ]]; then
        echo "Transfer error: Source and destination files have not the same size. Exiting..."
        exit 1
    fi

  done
  #if [[ ! -z $( echo $list_of_files_to_copy | grep splitted ) ]]; then
  #  join_cca_splitted_files
  #fi
}

set -xuve

# ec:/ptb/OSM_FORCING/forcing_ea_1_oper_1_1995* .
#for y in `seq 1979 2018`; do
for y in 2018; do
for m in `seq -w 01 12` ; do

f=forcing_ea_1_oper_1_${y}${m}00.tar.gz
echo ==== $y$m #$f
#[ ! -f $f ] && echo downloading $f && ecaccess-file-mget -retry 5 -force -progress ec:/ptb/OSM_FORCING/$f .
if [ ! -f $f ] ; then
    echo downloading $f 
    transfer_cca ec:/ptb/OSM_FORCING/$f .
else
    remote_size=$(ecaccess-file-size ec:/ptb/OSM_FORCING/$f)
    local_size=$(ls -lrt $f | awk '{print $5}')
    if [[ ${remote_size} != "No such file or directory" ]] && [[ ${remote_size} != "Permission denied" ]]  && [[ ${remote_size} != "ECFS not available (sleep)" ]] && [[ "$local_size" != "$remote_size" ]]; then
        echo "file $f wrong size: local=$local_size remote=$remote_size - downloading again!"
        transfer_cca ec:/ptb/OSM_FORCING/$f .
    fi
fi

done
done
